import { map, concatMap, filter } from 'rxjs/operators';
import { isObservable, Observable } from 'rxjs';
import { isPromise } from 'rxjs/internal-compatibility';

/**
 * Returns an Observable that emits a string character that is at the passed position in a source string using
 * String.charAt
 *
 * @category String Query
 *
 * @param position The index of the character to return in the source string
 *
 * @example
 * Return the character at index `1`
 * ```ts
 * of('RxJS Ninja').pipe(charAt(1)).subscribe();
 * ```
 * Output: `x`
 *
 * @returns Observable that emits a string character
 */

function charAt(position) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.charAt(position);
    }));
  };
}

/**
 * Returns an Observable that emits a number, the character code of a character at the passed position in a source
 * string using String.charCodeAt
 *
 * @category String Query
 *
 * @param position The index of the character to return in the source string
 *
 *
 * @example
 * Return the character code of the character at index `1`
 * ```ts
 * of('RxJS Ninja').pipe(charCodeAt(1)).subscribe();
 * ```
 * Output: `120`
 *
 * @returns Observable that emits a number that is a character code
 */

function charCodeAt(position) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.charCodeAt(position);
    }));
  };
}

/**
 * Returns an Observable that emits a number, the character code of a character at the passed position in a source
 * string using String.codePointAt
 *
 * @category String Query
 *
 * @param position The index of the character to return in the source string
 *
 * @example
 * Return the code point of the character at index `1`
 * ```ts
 * of('☃★♲').pipe(codePointAt(1)).subscribe();
 * ```
 * Output: `9733`
 *
 * @returns Observable that emits a number that is a code point
 */

function codePointAt(position) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.codePointAt(position);
    }));
  };
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __read(o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
}

function __spread() {
    for (var ar = [], i = 0; i < arguments.length; i++)
        ar = ar.concat(__read(arguments[i]));
    return ar;
}

/**
 * Returns an Observable that emits a string that is the source string concatenated with the passed input to the
 * operator using Sting.concat
 *
 * @category String Modify
 *
 * @param args Observable string source, array of strings or argument list of strings
 *
 * @example
 * Return a string that is a source appended with a list of strings
 * ```ts
 * of('RxJS').pipe(concat(' ', 'Ninja')).subscribe();
 * ```
 * Output: `RxJS Ninja`
 *
 * @example
 * Return a string that is a source appended with a array of strings
 * ```ts
 * of('RxJS').pipe(concat([' ', 'Ninja'])).subscribe();
 * ```
 * Output: `RxJS Ninja`
 *
 * @example
 * Return a string that is a source appended with an Observable string
 * ```ts
 * of('RxJS').pipe(concat(of([' ', 'Ninja']))).subscribe();
 * ```
 * Output: `RxJS Ninja`
 *
 * @returns Observable that emits a string
 */

function concat() {
  var args = [];

  for (var _i = 0; _i < arguments.length; _i++) {
    args[_i] = arguments[_i];
  }

  var values = __spread(args);

  if (values[0] instanceof Array) {
    return function (source) {
      return source.pipe(map(function (value) {
        return value.concat.apply(value, __spread(values[0]));
      }));
    };
  } else if (isObservable(values[0])) {
    return function (source) {
      return values[0].pipe(concatMap(function (strings) {
        return source.pipe(map(function (value) {
          return value.concat.apply(value, __spread(strings));
        }));
      }));
    };
  } else {
    return function (source) {
      return source.pipe(map(function (value) {
        return value.concat.apply(value, __spread(values));
      }));
    };
  }
}

/**
 * Returns an Observable that emits a boolean value where the source string ends with the passed string parameter using
 * String.endsWith
 *
 * @category String Query
 *
 * @see The [[filterEndsWith]] operator returns the string value
 *
 * @param ending The string to check the source ends with
 * @param length Optional length of the string to check
 *
 * @example
 * Return if the source string ends with the `S` character
 * ```ts
 * from(['RxJS', 'Ninja', 'Tests']).pipe(endsWith('S')).subscribe();
 * ```
 * Output: `true, false, false`
 *
 * @example
 * Return if the source string at up to length `4` ends with the `t` character
 * ```ts
 * from(['RxJS', 'Ninja', 'Tests']).pipe(endsWith('t', 4)).subscribe();
 * ```
 * Output: `false, false, true`
 *
 * @returns Observable that emits a boolean of the source string ending with the passed input
 */

function endsWith(ending, length) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.endsWith(ending, length);
    }));
  };
}

/**
 * Returns an Observable that emits a string where the source string ends with the passed ending using String.endsWith
 *
 * @category String Filter
 *
 * @see The [[endsWith]] operator returns the boolean value
 *
 * @param ending The string to check the source ends with
 * @param length Optional length of the string to check
 *
 * @example
 * Return a string where the source string ends with the `S` character
 * ```ts
 * from(['RxJS', 'Ninja', 'Tests']).pipe(filterEndsWith('S')).subscribe();
 * ```
 * Output: `RxJS`
 *
 * @example
 * Return a string where the source string at up to length `4` ends with the `t` character
 * ```ts
 * from(['RxJS', 'Ninja', 'Tests']).pipe(filterEndsWith('t', 4)).subscribe();
 * ```
 * Output: `Tests`
 *
 * @returns Observable that emits a string
 */

function filterEndsWith(ending, length) {
  return function (source) {
    return source.pipe(filter(function (value) {
      return value.endsWith(ending, length);
    }));
  };
}

/**
 * Returns an Observable that emits a string where the source string contains with the passed search string using
 * String.includes
 *
 * @category String Filter
 *
 * @see The [[includes]] operator returns the boolean value
 *
 * @param searchStr The string to check the source ends with
 *
 * @example
 * Return a string where the source string includes 'JS'
 * ```ts
 * from(['RxJS', 'Ninja', 'Tests']).pipe(filterIncludes('JS')).subscribe();
 * ```
 * Output: `RxJS`
 *
 * @returns Observable that emits a string
 */

function filterIncludes(searchStr) {
  return function (source) {
    return source.pipe(filter(function (value) {
      return value.includes(searchStr);
    }));
  };
}

/**
 * Returns an Observable that emits a string where the source string starts with the passed string using
 * String.startsWith
 *
 * @category String Filter
 *
 * @see The [[startsWith]] operator returns the boolean value
 *
 * @param start The string to check the source starts with
 * @param startFrom Optional index to start the check from
 *
 * @example
 * Return a string where the source string starts with the `N` character
 * ```ts
 * from(['RxJS', 'Ninja', 'Tests']).pipe(filterStartsWith('N')).subscribe();
 * ```
 * Output: `Ninja`
 *
 * @example
 * Return a string where the source string starts with `t` character from index `3`
 * ```ts
 * from(['RxJS', 'Ninja', 'Tests']).pipe(filterStartsWith('t', 3)).subscribe();
 * ```
 * Output: `Tests`
 *
 * @returns Observable that emits a string
 */

function filterStartsWith(start, startFrom) {
  return function (source) {
    return source.pipe(filter(function (value) {
      return value.startsWith(start, startFrom);
    }));
  };
}

/**
 * Returns an Observable that emits a string made from character codes using String.fromCharCode
 *
 * @remarks This operator will emit a single string for all input passed including arrays
 *
 * @see The [[mapCharCode]] operator can be used to map an Observable source of char codes to strings
 *
 * @category String Observables
 *
 * @param charCodes Observable input, Promise, Array or argument list of character codes
 *
 * @example
 * Return a string from character code arguments
 * ```ts
 * fromCharCode(82, 120, 74, 83).subscribe();
 * ```
 * Output: `RxJS`
 *
 * @example
 * Return a string from character code array
 * ```ts
 * fromCharCode([82, 120, 74, 83]).subscribe();
 * ```
 * Output: `RxJS`
 *
 * @example
 * Return a string from character code Observable
 * ```ts
 * fromCharCode(of([82, 120, 74, 83]).subscribe();
 * ```
 * Output: `RxJS`
 *
 * @returns Observable that emits a string
 */

function fromCharCode() {
  var charCodes = [];

  for (var _i = 0; _i < arguments.length; _i++) {
    charCodes[_i] = arguments[_i];
  }

  if (isObservable(charCodes[0])) {
    return charCodes[0].pipe(map(function (input) {
      return String.fromCharCode.apply(String, __spread(input));
    }));
  } else if (isPromise(charCodes[0])) {
    return new Observable(function (subscriber) {
      charCodes[0].then(function (input) {
        if (!subscriber.closed) {
          subscriber.next(String.fromCharCode.apply(String, __spread(input)));
          subscriber.complete();
        }
      }, function (err) {
        return subscriber.error(err);
      });
    });
  } else {
    var value_1 = Array.isArray(charCodes[0]) ? charCodes[0] : __spread(charCodes);
    return new Observable(function (subscriber) {
      subscriber.next(String.fromCharCode.apply(String, __spread(value_1)));
      subscriber.complete();
    });
  }
}

/**
 * Returns an Observable that emits a string made from code points using String.fromCodePoint
 *
 * @remarks This operator will emit a single string for all input passed including arrays
 *
 * @category String Observables
 *
 * @param charCodes Observable input, Promise, Array or argument list of code points
 *
 * @example
 * Return a string from code points arguments
 * ```ts
 * fromCodePoint(9731, 9733, 9842).subscribe();
 * ```
 * Output: `☃★♲`
 *
 * @example
 * Return a string from code points array
 * ```ts
 * fromCodePoint([9731, 9733, 9842]).subscribe();
 * ```
 * Output: `☃★♲`
 *
 * @example
 * Return a string from code points Observable
 * ```ts
 * fromCodePoint(of([9731, 9733, 9842]).subscribe();
 * ```
 * Output: `☃★♲`
 *
 * @returns Observable that emits a string
 */

function fromCodePoint() {
  var charCodes = [];

  for (var _i = 0; _i < arguments.length; _i++) {
    charCodes[_i] = arguments[_i];
  }

  if (isObservable(charCodes[0])) {
    return charCodes[0].pipe(map(function (input) {
      return String.fromCodePoint.apply(String, __spread(input));
    }));
  } else if (isPromise(charCodes[0])) {
    return new Observable(function (subscriber) {
      charCodes[0].then(function (input) {
        if (!subscriber.closed) {
          subscriber.next(String.fromCodePoint.apply(String, __spread(input)));
          subscriber.complete();
        }
      }, function (err) {
        return subscriber.error(err);
      });
    });
  } else {
    var value_1 = Array.isArray(charCodes[0]) ? charCodes[0] : __spread(charCodes);
    return new Observable(function (subscriber) {
      subscriber.next(String.fromCodePoint.apply(String, __spread(value_1)));
      subscriber.complete();
    });
  }
}

/**
 * Returns an Observable that emits strings from any an argument list of strings or  supported Observable, Promise or
 * Array-like source
 *
 * @category String Observables
 *
 * @remarks This operator is a type-safe {@link https://rxjs.dev/api/index/function/from|from} and will emit only
 * strings, also unlike `from` a single string is not converted into an array-like.
 *
 * @param input Argument list, Observable input, Promise or Array of strings
 *
 * @example
 * Return a reversed string from an argument list of strings
 * ```ts
 * fromString('RxJS', 'Ninja').pipe(reverse()).subscribe();
 * ```
 * Output: `'SJxR', 'ajniN'`
 *
 * @example
 * Return a reversed string from an array list of strings
 * ```ts
 * fromString(['RxJS', 'Ninja']).pipe(reverse()).subscribe();
 * ```
 * Output: `'SJxR', 'ajniN'`
 *
 * @example
 * Return a reversed string from an Observable array list of strings
 * ```ts
 * fromString(of(['RxJS', 'Ninja'])).pipe(reverse()).subscribe();
 * ```
 * Output: `'SJxR', 'ajniN'`
 *
 * @returns Observable that emits a string
 */

function fromString() {
  var input = [];

  for (var _i = 0; _i < arguments.length; _i++) {
    input[_i] = arguments[_i];
  }

  if (isObservable(input[0])) {
    return input[0];
  } else if (isPromise(input[0])) {
    return new Observable(function (subscriber) {
      input[0].then(function (value) {
        if (!subscriber.closed) {
          subscriber.next(value);
          subscriber.complete();
        }
      }, function (err) {
        return subscriber.error(err);
      });
    });
  } else {
    var value_1 = Array.isArray(input[0]) ? input[0] : __spread(input);
    return new Observable(function (subscriber) {
      for (var i = 0; i < value_1.length; i++) {
        subscriber.next(value_1[i]);
      }

      subscriber.complete();
    });
  }
}

/**
 * Returns an Observable that emits a string made from a source unicode string using String.normalize
 *
 * @category String Observables
 *
 * @remarks This operator is a type-safe {@link https://rxjs.dev/api/index/function/from|from} and will emit only
 * strings, also unlike `from` a single string is not converted into an array-like.
 *
 * @param input A string or array of string unicode characters
 * @param form The Unicode Normalization Form to decode the string with
 *
 * @example
 * Returns a string from Unicode characters
 * ```ts
 * fromUnicode('\u0041\u006d\u00e9\u006c\u0069\u0065').subscribe();
 * ```
 * Output: `Amélie`
 *
 * @returns Observable that emits a string
 */

function fromUnicode(input, form) {
  if (isObservable(input)) {
    return input.pipe(map(function (value) {
      return String.fromCodePoint.apply(String, __spread(value));
    }));
  } else if (isPromise(input)) {
    return new Observable(function (subscriber) {
      input.then(function (value) {
        if (!subscriber.closed) {
          subscriber.next(String.fromCodePoint.apply(String, __spread(value)));
          subscriber.complete();
        }
      }, function (err) {
        return subscriber.error(err);
      });
    });
  } else {
    var value_1 = Array.isArray(input) ? input : [input];
    return new Observable(function (subscriber) {
      for (var i = 0; i < value_1.length; i++) {
        subscriber.next(value_1[i].normalize(form));
      }

      subscriber.complete();
    });
  }
}

/**
 * Returns an Observable that emits a boolean where the source string contains with the passed search string using
 * String.includes
 *
 * @category String Query
 *
 * @see The [[filterIncludes]] operator returns the string value
 *
 * @param searchStr The string to check the source ends with
 *
 * @example
 * Return a boolean where the source string includes 'JS'
 * ```ts
 * from(['RxJS', 'Ninja', 'Tests']).pipe(includes('JS')).subscribe();
 * ```
 * Output: `true, false, false`
 *
 * @returns Observable that emits a boolean
 */

function includes(searchStr) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.includes(searchStr);
    }));
  };
}

/**
 * Returns an Observable that emits a number of the first index from the source string where the search string begins
 * using String.indexOf
 *
 * @category String Query
 *
 * @param searchStr The string to search in the source string
 * @param start Optional start position if not from the beginning of the string
 *
 * @example
 * Returns the first index of `RxJS` in the string
 * ```ts
 * of('Hello RxJS Ninja, RxJS Rocks').pipe(indexOf('RxJS')).subscribe();
 * ```
 * Output: `6`
 *
 * @example
 * Returns the first index of `RxJS` in the string, starting from index `8`
 * ```ts
 * of('Hello RxJS Ninja, RxJS Rocks').pipe(indexOf('RxJS', 8)).subscribe();
 * ```
 * Output: `18`
 *
 * @returns Observable that emits a number that is the first index of the search string in the source string
 */

function indexOf(searchStr, start) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.indexOf(searchStr, start);
    }));
  };
}

/**
 * Returns an Observable that emits a number of the last index from the source string where the search string begins
 * using String.lastIndexOf
 *
 * @category String Query
 *
 * @param searchStr The string to search in the source string
 * @param lastIndex The index of the last character in the string to search up to
 *
 * @example
 * Returns the first index of `RxJS` in the string
 * ```ts
 * of('Hello RxJS Ninja, RxJS Rocks').pipe(lastIndexOf('RxJS')).subscribe();
 * ```
 * Output: `18`
 *
 * @example
 * Returns the first index of `RxJS` in the string, ending at index `15`
 * ```ts
 * of('Hello RxJS Ninja, RxJS Rocks').pipe(lastIndexOf('RxJS', 15)).subscribe();
 * ```
 * Output: `6`
 *
 * @returns Observable that emits a number that is the last index of the search string in the source string
 */

function lastIndexOf(searchStr, lastIndex) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.lastIndexOf(searchStr, lastIndex);
    }));
  };
}

/**
 * Returns an Observable that emits a string from a source of character codes using String.fromCharCode
 *
 * @category String Mapping
 *
 * @see The [[fromCharCode]] Observable can be used to generate a string source from character codes
 *
 * @example
 * Returns a string `RxJS` from a source array of character codes
 * ```ts
 * of([82, 120, 74, 83]).pipe(mapCharCode()).subscribe()
 * ```
 * Output: `RxJS`
 *
 *
 * @returns Observable that emits a string from source character codes
 */

function mapCharCode() {
  return function (source) {
    return source.pipe(map(function (values) {
      return Array.isArray(values) ? String.fromCharCode.apply(String, __spread(values)) : String.fromCharCode(values);
    }));
  };
}

/**
 * Returns an Observable that emits a string from a source of character codes using String.fromCodePoint
 *
 * @category String Mapping
 *
 * @see The [[fromCodePoint]] Observable can be used to generate a string source from character codes
 *
 * @example
 * Returns a string `RxJS` from a source array of code points
 * ```ts
 * of([9731, 9733, 9842]).pipe(mapCodePoint()).subscribe()
 * ```
 * Output: `☃★♲`
 *
 * @returns Observable that emits a string from source code points
 */

function mapCodePoint() {
  return function (source) {
    return source.pipe(map(function (values) {
      return Array.isArray(values) ? String.fromCodePoint.apply(String, __spread(values)) : String.fromCodePoint(values);
    }));
  };
}

/**
 * Returns an Observable that emits a `RegExpMatchArray` where a source string returns a valid result using using
 * String.match. If no result is found, `null` is emitted.
 *
 * @category String Query
 *
 * @param pattern A string or RegExp to match
 *
 * @example
 * Returns a single match for the string `RxJS` in the source string
 * ```ts
 * of('Hello RxJS Ninja, RxJS Rocks').pipe(match('RxJS')).subscribe();
 * ```
 * Output: `['RxJS', index: 6, input: 'Hello RxJS Ninja, RxJS Rocks', groups: undefined]`
 *
 * @example
 * Returns a match with a RegExp with global `/g` for `RxJS`
 * ```ts
 * of('Hello RxJS Ninja, RxJS Rocks').pipe(match(/RxJS/g)).subscribe();
 * ```
 * Output: `['RxJS', 'RxJS']`
 *
 * @returns Observable that emits a RegExpMatchArray
 */

function match(pattern) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.match(pattern);
    }));
  };
}

/**
 * Returns an Observable that emits an array of results from String.matchAll
 *
 * @category String Query
 *
 * @remarks This operator converts the `IterableIterator<RegExpMatchArray>` to `RegExpMatchArray[]` to avoid dealing
 *   with iterators.
 *
 * @param pattern A RegExp regular expression to match in the string
 *
 * @example
 * Returns all matching groups in the source string
 * ```ts
 * of('RxJS Ninja,Hello Ninja').pipe(matchAll(/([A-Za-z\s]*)Ninja/g).subscribe();
 * ```
 *
 * Output:`["RxJS Ninja", index: 0, input: "RxJS Ninja,Hello Ninja"], ["Hello Ninja", index: 11, input: "RxJS
 *   Ninja,Hello Ninja"]`
 *
 * @returns Observable that emits an array of RegExpMatchArray
 */

function matchAll(pattern) {
  return function (source) {
    return source.pipe(map(function (value) {
      return __spread(value.matchAll(pattern));
    }));
  };
}

/**
 * Returns an Observable that emits a string made from a source unicode string using String.normalize
 *
 * @category String Mapping
 *
 * @param form The Unicode Normalization Form to decode the string with
 *
 * @example
 * Returns a string from Unicode characters
 * ```ts
 * of('\u0041\u006d\u00e9\u006c\u0069\u0065').pipe(normalize()).subscribe();
 * ```
 * Output: `Amélie`
 *
 * @returns Observable that emits a string
 */

function normalize(form) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.normalize(form);
    }));
  };
}

/**
 * Returns an Observable that emits a string where the source string has been padded using String.padStart
 *
 * @category String Modify
 *
 * @alias `padLeft`
 *
 * @param maxLength The maximum length to pad the string to
 * @param fillString Optional string to use as the string padding
 *
 * @example
 * Returns a string padded to a length of `12` with default fill string
 * ```ts
 * of('RxJS Ninja').pipe(padStart(12)).subscribe();
 * ```
 * Output: `'  RxJS Ninja'`
 *
 *
 * @example
 * Returns a string padded to a length of `12` with `.` fill string
 * ```ts
 * of('12345').pipe(padStart(12, '.')).subscribe();
 * ```
 * Output: `'..RxJS Ninja'`
 *
 * @returns Observable that emits a string that is padded to the passed length
 */

function padStart(maxLength, fillString) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.padStart(maxLength, fillString);
    }));
  };
}
/**
 * Returns an Observable that emits a string where the source string has been padded using String.padEnd
 *
 * @category String Modify
 *
 * @alias `padRight`
 *
 * @param maxLength The maximum length to pad the string to
 * @param fillString Optional string to use as the string padding
 *
 * @example
 * Returns a string padded to a length of `12` with default fill string
 * ```ts
 * of('RxJS Ninja').pipe(padEnd(12)).subscribe();
 * ```
 * Output: `'RxJS Ninja  '`
 *
 *
 * @example
 * Returns a string padded to a length of `12` with `.` fill string
 * ```ts
 * of('12345').pipe(padEnd(12, '.')).subscribe();
 * ```
 * Output: `'RxJS Ninja..'`
 *
 * @returns Observable that emits a string that is padded to the passed length
 */

function padEnd(maxLength, fillString) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.padEnd(maxLength, fillString);
    }));
  };
}

/**
 * Returns an Observable that emits a string where the source string is repeated with String.repeat.
 *
 * @remarks If a separator is passed it uses an array and will join the result using the separator instead, as
 *   `String.repeat` does not support it
 *
 * @category String Modify
 *
 * @param count The number of times to repeat the string
 * @param separator Optional separator for joining strings
 *
 * @example
 * Returns a string with the word `Ninja` repeated 5 times
 * ```ts
 * of('Ninja').pipe(repeat(5)).subscribe();
 * ```
 * Output: `NinjaNinjaNinjaNinjaNinja`
 *
 * @example
 * Returns a string with the word `Ninja` repeated 5 times with a `,` separator
 * ```ts
 * of('Ninja').pipe(repeat(5, ', ')).subscribe();
 * ```
 * Output: `Ninja, Ninja, Ninja, Ninja, Ninja`
 *
 * @returns Observable that emits a string of the source string repeated
 */

function repeat(count, separator) {
  return separator ? function (source) {
    return source.pipe(map(function (value) {
      return new Array(count).fill(value).join(separator);
    }));
  } : function (source) {
    return source.pipe(map(function (value) {
      return value.repeat(count);
    }));
  };
}

/**
 * Returns an Observable that emits a string, replacing text in the source string with the replacement text if the
 * pattern is found using String.replace
 *
 * @category String Modify
 *
 * @param pattern A string or RegExp to find in the Observable string to replace
 * @param replacement The replacement string
 *
 * @example
 * Return a string with first instance `Hero` replaced with `Ninja`
 * ```ts
 * of('RxJS Hero, Angular Hero').pipe(replace('Hero', 'Ninja')).subscribe();
 * ```
 * Output: `RxJS Ninja, Angular Hero`
 *
 * @example
 * Return a string where `Hero` is replaced with `Ninja` using a RegExp
 * ```ts
 * of('RxJS Hero, Angular Hero').pipe(replace(/(?!\w+\s)(\w+)/, 'Ninja')).subscribe();
 * ```
 * Output: `RxJS Ninja, Angular Hero`
 *
 * @returns Observable that emits a string
 */

function replace(pattern, replacement) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.replace(pattern, replacement);
    }));
  };
}

/**
 * Returns an Observable that emits a string, replacing text in the source string with the replacement text if the
 * pattern is found using String.replaceAll
 *
 * @category String Modify
 *
 * @param pattern A string or RegExp to find in the Observable string to replace
 * @param replacement The replacement string
 *
 * @example
 * Return a string with all instances of `Hero` replaced with `Ninja`
 * ```ts
 * of('RxJS Hero, Angular Hero').pipe(replaceAll('Hero', 'Ninja')).subscribe();
 * ```
 * Output: `RxJS Ninja, Angular Ninja`
 *
 * @example
 * Return a string where `Hero` is replaced with `Ninja` using a RegExp
 * ```ts
 * of('RxJS Hero, Angular Hero').pipe(replaceAll(/(?!\w+\s)(\w+)/g, 'Ninja')).subscribe();
 * ```
 * Output: `RxJS Ninja, Angular Ninja`
 *
 * @returns Observable that emits a string
 */

function replaceAll(pattern, replacement) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.replaceAll(pattern, replacement);
    }));
  };
}

/**
 * Returns an Observable that emits a string where the source string is reversed.
 *
 * @category String Modify
 *
 * @remarks This operator turns a string into an array and uses Array.reverse and Array.join to create
 * a new string
 *
 * @example
 * Returns a string that is reversed
 * ```ts
 * of('emordnilaP').pipe(reverse()).subscribe();
 * ```
 * Output: `'Palindrome'`
 *
 * @returns Observable that emits a string that is reversed from the source
 */

function reverse() {
  return function (source) {
    return source.pipe(map(function (value) {
      return __spread(value).reverse().join('');
    }));
  };
}

/**
 * Returns an Observable that emits a number that is the first index of where the value is found using String.search
 *
 * @category String Query
 *
 * @param pattern A string or RegExp to match in the string
 *
 * @example
 * Return the first index of `Ninja`
 * ```ts
 * of('RxJS Ninja, Angular Ninja').pipe(search('Ninja')).subscribe();
 * ```
 * Output: `5`
 *
 * @example
 * Return the first index of the first found group from the RegExp
 * ```ts
 * of('RxJS Ninja, Angular Ninja').pipe(search(/(?!\w+\s)(\w+)/g)).subscribe();
 * ```
 * Output: `5`
 *
 * @returns Observable that emits an number that is the start index of the first found value
 */

function search(pattern) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.search(pattern);
    }));
  };
}

/**
 * Returns an Observable that emits a string that is a partial slice of the source string using String.slice
 *
 * @category String Modify
 *
 * @remarks For creating substrings use [[substring]] as a better alternative to `slice`
 *
 * @param startIndex The start index for the substring
 * @param endIndex Optional end index for the length of substring, if not passed slice will use `str.length -1`
 *
 * @example
 * Return a string from index `0` to `4`
 * ```ts
 * of('RxJS Ninja').pipe(slice(0, 4)).subscribe();
 * ```
 * Output: `RxJS`
 *
 * @example
 * Return a string from index `5` to the end of the string
 * ```ts
 * of('RxJS Ninja').pipe(slice(5)).subscribe();
 * ```
 * Output: `Ninja`
 *
 * @returns Observable that emits a string that is a slice of the source string
 */

function slice(startIndex, endIndex) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.slice(startIndex, endIndex);
    }));
  };
}

/**
 * Returns an Observable that emits an array from a source string split by the separator using String.split
 *
 * @category String to Array
 *
 * @param separator The character to split the string at
 * @param limit Optional limit for the number of times to split
 *
 * @example
 * Returns an array of a string split on the `|` character
 * ```ts
 * of('Hello|World|RxJS|Ninja').pipe(split('|')).subscribe();
 * ```
 * Output: `['Hello', 'World', 'RxJS', 'Ninja']`
 *
 * @example
 * Returns an array of a string split on the `|` character, limiting to the first `2`
 * ```ts
 * of('Hello|World|RxJS|Ninja').pipe(split('|', 2)).subscribe();
 * ```
 * Output: `['Hello', 'World']`
 *
 * @returns Observable that emits an array of strings from the source string split on the separator
 */

function split(separator, limit) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.split(separator, limit);
    }));
  };
}

/**
 * Returns an Observable that emits a boolean when the source string contains the input string at the start of the
 * source string using String.startsWith
 *
 * @category String Query
 *
 * @param input The string to check the source string start with
 * @param startIndex Optional start index to being searching the string from
 *
 * @remarks
 * If you need to get the string value instead of boolean use [[filterStartsWith]]
 *
 * @example
 * Return a boolean if the source string starts with the passed input
 * ```ts
 * from(['RxJS', 'Ninja', 'Hero']).pipe(startsWith('R')).subscribe();
 * ```
 * Output: `true, false, false`
 *
 * @example
 * Return a boolean if the source string starts with the passed input from index `1`
 * ```ts
 * from(['RxJS', 'Ninja', 'Hero']).pipe(startsWith('x', 1)).subscribe();
 * ```
 * Output: `true, false, false`
 *
 * @returns Observable that emits a boolean if the source string start with the input string
 */

function startsWith(input, startIndex) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.startsWith(input, startIndex);
    }));
  };
}

/**
 * Returns an Observable that emits a string that is a partial slice of the source string using String.substring
 *
 * @category String Modify
 *
 * @param indexStart The index of the first character to include in the returned substring.
 * @param indexEnd Optional The index of the first character to exclude from the returned substring.
 *
 * @example
 * Return a string from index `0` to `4`
 * ```ts
 * of('RxJS Ninja').pipe(substring(0, 4)).subscribe();
 * ```
 * Output: `RxJS`
 *
 * @example
 * Return a string from index `5` to the end of the string
 * ```ts
 * of('RxJS Ninja').pipe(substring(5)).subscribe();
 * ```
 * Output: `Ninja`
 *
 * @returns Observable that emits a string that is a substring of the source string
 */

function substring(indexStart, indexEnd) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.substring(indexStart, indexEnd);
    }));
  };
}

/**
 * Default words to exclude when using the [[titleize]] operator
 */

var NO_CAP_WORDS = ['a', 'and', 'but', 'is', 'or', 'over', 'the', 'to', 'too'];
/**
 * Return an Observable that emits a string where the source string is titleized (first letter of each word uppercase).
 * The operator uses String.toLocaleUpperCase so can support locale strings
 *
 * By default is uses the [[NO_CAP_WORDS]] to skip some words (unless they are the first word in the string).
 * To disable this pass an empty array, and to extend you can use the default with a spread operator to extend:
 * `[...NO_CAP_WORDS, 'foo', 'bar']`. When using a separator, if no change in default pass `undefined`
 *
 * It will also skip words already starting with a capital (e.g. `RxJS`)
 *
 * @category String Modify
 *
 * @param noTitleWords A list of words to exclude from making a title word
 * @param separator Optional separator to use when joining each word
 * @param locales Locales for string formatting
 *
 * @example
 * Returns a string titilzed with default no caps words
 * ```ts
 * of('the RxJS ninja').pipe(titleize()).subscribe();
 * ```
 * Output: `The RxJS Ninja`
 *
 * @example
 * Returns a string titilzed with additional no cap words
 * ```ts
 * of('the RxJS ninja').pipe(titleize([...NO_CAP_WORDS, 'ninja'])).subscribe();
 * ```
 * Output: `The RxJS ninja`
 *
 * @example
 * Returns a string titilzed with a separator
 * ```ts
 * of('angular,RxJS,typescript,ninja').pipe(titleize(undefined, ',')).subscribe();
 * ```
 * Output: `Angular,RxJS,Typescript,Ninja`
 *
 * @example
 * Returns a string titilzed with a different locale
 * ```ts
 * of('ängulär,RxJS,typescript,ninjä').pipe(titleize(undefined, undefined, 'de-DE')).subscribe();
 * ```
 * Output: `Ängulär,RxJS,Typescript,Ninjä`
 *
 * @returns Observable that emits a titilzed string
 */

function titleize(noTitleWords, separator, locales) {
  if (noTitleWords === void 0) {
    noTitleWords = NO_CAP_WORDS;
  }

  if (separator === void 0) {
    separator = ' ';
  }

  return function (source) {
    return source.pipe(split(separator), map(function (values) {
      return values.map(function (word, index) {
        if (index && noTitleWords.includes(word)) {
          return word;
        }

        return word.charCodeAt(0) >= 65 && word.charCodeAt(0) <= 90 ? word : "" + word.charAt(0).toLocaleUpperCase(locales) + word.slice(1);
      }).join(separator);
    }));
  };
}

/**
 * Returns an Observable that emits a string where the source string is passed through String.toLocaleLowerCase
 *
 * @category String Modify
 *
 * @param locales Optional locales to pass for string formatting
 *
 * @example
 * Returns an lower case string
 * ```ts
 * of('APPLE').pipe(toLowerCase()).subscribe();
 * ```
 * Output: `'apple'`
 *
 * @example
 *  * Returns an lower case string with locale
 * ```ts
 * of('ÄPFEL').pipe(toLowerCase('de-DE')).subscribe();
 * ```
 * Output: `'äpfel'`
 *
 * @returns Observable that emits a lower case string
 */

function toLowerCase(locales) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.toLocaleLowerCase(locales);
    }));
  };
}

/**
 * Returns an Observable that emits a string where the source string is passed through String.toLocaleLowerCase
 *
 * @category String Modify
 *
 * @param locales Optional locales to pass for string formatting
 *
 * @example
 * Returns an upper case string
 * ```ts
 * of('apple').pipe(toUpperCase()).subscribe();
 * ```
 * Output: `'APPLE'`
 *
 * @example
 * Returns an lower case string with locale
 * ```ts
 * of('äpfel').pipe(toUpperCase('de-DE')).subscribe();
 * ```
 * Output: `'ÄPFEL'`
 *
 * @returns Observable that emits a lower case string
 */

function toUpperCase(locales) {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.toLocaleUpperCase(locales);
    }));
  };
}

/**
 * Returns an Observable that emits a string where the source string has any white space at the start removed using
 * String.trimStart
 *
 * @category String Modify
 *
 * @alias `trimLeft`
 *
 * @example
 * Returns the source string with any whitespace at the start removed
 * ```ts
 * of('  RxJS Ninja  ').pipe(trimStart()).subscribe();
 * ```
 * Output: `'RxJS Ninja  '`
 *
 * @returns Observable that emits a trimmed string
 */

function trimStart() {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.trimStart();
    }));
  };
}
/**
 * Returns an Observable that emits a string where the source string has any white space at the end removed using
 * String.trimEnd
 *
 * @category String Modify
 *
 * @alias `trimRight`
 *
 * @example
 * Returns the source string with any whitespace at the end removed
 * ```ts
 * of('  RxJS Ninja  ').pipe(trimStart()).subscribe();
 * ```
 * Output: `'  RxJS Ninja'`
 *
 * @returns Observable that emits a trimmed string
 */

function trimEnd() {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.trimEnd();
    }));
  };
}
/**
 * Returns an Observable that emits a string where the source string has any white space at the ends removed using
 * String.trim
 *
 * @category String Modify
 *
 * @example
 * Returns the source string with any whitespace at both ends removed
 * ```ts
 * of('  RxJS Ninja  ').pipe(trimStart()).subscribe();
 * ```
 * Output: `'RxJS Ninja'`
 *
 * @returns Observable that emits a trimmed string
 */

function trim() {
  return function (source) {
    return source.pipe(map(function (value) {
      return value.trim();
    }));
  };
}

/**
 * @packageDocumentation
 * @module String
 */

/**
 * The type of form to normalize a unicode string with String.normalize, default is 'NFC'
 */
var FormType;

(function (FormType) {
  /**
   * Canonical Decomposition, followed by Canonical Composition.
   */
  FormType["NFC"] = "NFC";
  /**
   * Canonical Decomposition.
   */

  FormType["NFD"] = "NFD";
  /**
   * Compatibility Decomposition, followed by Canonical Composition.
   */

  FormType["NFKC"] = "NFKC";
  /**
   * Compatibility Decomposition.
   */

  FormType["NFKD"] = "NFKD";
})(FormType || (FormType = {}));

export { FormType, NO_CAP_WORDS, charAt, charCodeAt, codePointAt, concat, endsWith, filterEndsWith, filterIncludes, filterStartsWith, fromCharCode, fromCodePoint, fromString, fromUnicode, includes, indexOf, lastIndexOf, mapCharCode, mapCodePoint, match, matchAll, normalize, padEnd, padStart as padLeft, padEnd as padRight, padStart, repeat, replace, replaceAll, reverse, search, slice, split, startsWith, substring, titleize, toLowerCase, toUpperCase, trim, trimEnd, trimStart as trimLeft, trimEnd as trimRight, trimStart };
